# -*- coding: utf-8 -*-
# moebelModell.py
## die erste Variante (ohne Mausereignisse)

"""Eine eigene Modellklasse für die Moebel-Klassen
übernimmt die gesamte Verwaltung der erzeugten Möbel
einschließlich Speichern und Laden,
sowie Auswahl
Die Klasse ist als Singleton modelliert"""

import pickle # Persistenz realisieren

from badewanne import Badewanne
from bett import Bett
from dusche import Dusche
from herd import Herd
from klosett import Klosett
from sessel import Sessel
from schrank import Schrank
from sofa import Sofa
from stuhl import Stuhl
from tisch import Tisch
from ovalerTisch import OvalerTisch

### ---------------------------------------------------
class MoebelModell(object):
    '''Ist als Singleton modelliert'''

    __moebelModell = None

    @staticmethod
    def GibMoebelModell():
        if MoebelModell.__moebelModell == None:
            MoebelModell.__moebelModell=MoebelModell()
        return MoebelModell.__moebelModell
    
    def __init__(self):
        """Konstruktor"""
        ## Fuer Singleton: ----------------------------------------
        if MoebelModell.__moebelModell != None:
            raise Exception('Konstruktor nicht direkt aufrufen')
        ## --------------------------------------------------------
        self.__alleMoebel = []
        self.__ausgewaehlt = -1

    #### ----- Erzeugen -----
    def Neu(self, klasse, *par, **kwargs):
        if klasse.__class__==str:
            klasse=eval(klasse)
        neu=klasse(*par, **kwargs)
        self.__alleMoebel.append(neu)
        return neu

    # Loeschen s.u.

    #### ----- sichern und laden -----
    def Speichere(self, dateiname):
        """Objektdaten werden in Datei geschrieben"""
        try:
            with open(dateiname, 'wb') as objekt_datei:
                pickle.dump(self.__alleMoebel, objekt_datei)
                return 'OK'
        except IOError as ioerr:
            return 'Dateifehler: ' + str(ioerr)
        except pickle.PicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def Lade(self, dateiname):
        """Objektdaten werden aus der Datei gelesen"""
        for moebel in self.__alleMoebel:
            moebel.Verberge()
            del(moebel)
        self.__ausgewaehlt = -1
        try:
            with open(dateiname, 'rb') as objekt_datei:
                self.__alleMoebel = pickle.load(objekt_datei)
                for moebel in self.__alleMoebel:
                    if moebel.GibSichtbar():
                        moebel.Zeige()
                return 'OK'
        except IOError:
            return 'Datei nicht auffindbar!'
        except AttributeError as ae:
            return 'AttributeError: ' + str(ae)
        except pickle.UnpicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def BildExport(self, dateiname):
        """Bild wird in Bild-Datei geschrieben"""
        Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)            

    #### ----- Auswahl -----
    def Waehle(self,index=None):
        '''wählt das Moebel indexbasiert aus;
        None zum Weitersetzen der Auswahl, -1 zum Beenden der Auswahl,
        nach dem letzten wird die Auswahl ebenfalls beendet'''
        if len(self.__alleMoebel)==0: # nichts auszuwaehlen
            return
        if index==-1: # Beenden der Auswahl
            aktuell = self.__ausgewaehlt
            self.__ausgewaehlt = -1
            if aktuell!=-1: # Darstellung aktualisieren
                self.__alleMoebel[aktuell].Waehle(False)
                self.__alleMoebel[aktuell].Zeige()
            return
        if index!=None: # Indexfehler pruefen
            if index<-1: return
            if index>=len(self.__alleMoebel): return
            ## sonst auswaehlen und markieren
            aktuell = self.__ausgewaehlt
            self.__ausgewaehlt = index
            if aktuell!=-1: # Auswahl loeschen
                self.__alleMoebel[aktuell].Waehle(False)
            self.__alleMoebel[self.__ausgewaehlt].Waehle(True)
            self.__alleMoebel[aktuell].Zeige()
        if index==None: # Weitersetzen
            if self.__ausgewaehlt!=-1:
                self.__alleMoebel[self.__ausgewaehlt].Waehle(False)
                self.__alleMoebel[self.__ausgewaehlt].Zeige()                
            self.__ausgewaehlt+=1
            if self.__ausgewaehlt==len(self.__alleMoebel):
                self.__ausgewaehlt=-1
                return
            self.__alleMoebel[self.__ausgewaehlt].Waehle(True)
            self.__alleMoebel[self.__ausgewaehlt].Zeige()                

    def GibAusgewaehltes(self):
        if self.__ausgewaehlt<0: return None
        else: return self.__alleMoebel[self.__ausgewaehlt]

    def LoescheAusgewaehltes(self):
        if self.__ausgewaehlt>=0:
            self.GibAusgewaehltes().Verberge()
            self.__alleMoebel.remove(self.GibAusgewaehltes())
            self.__ausgewaehlt=-1

    #### ----- Aktionen -----
    def BewegeHorizontal(self, weite):
        """Bewegt das Ausgewaehlte horizontal
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.BewegeHorizontal(weite)
        
    def BewegeVertikal(self, weite):
        """Bewegt das Ausgewaehlte vertikal
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.BewegeVertikal(weite)
        
    def Drehe(self, winkel):
        """Dreht das Ausgewaehlte
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.Drehe(winkel)
        
    def AendereFarbe(self, neueFarbe):
        """Dreht das Ausgewaehlte
        reicht die Anforderung ggf weiter"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.AendereFarbe(neueFarbe)

    def GibAlleMoebel(self):
        return self.__alleMoebel

### ---------------------------------------------------
import wx
from grafikfenster import *
class MoebelApp(wx.App):
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.modell=MoebelModell.GibMoebelModell()
        ##self.TestAnwendung()
        return True

    def TestAnwendung(self):
        global modell, schrank, sessel, stuhl, tisch
        modell=self.modell
        schrank=self.modell.Neu(Schrank,280, 40, 100, 60, 90, 'black', True)
        stuhl=self.modell.Neu(Stuhl,10, 30, 40, 40, 270, 'blue', True)
        tisch=self.modell.Neu(Tisch,60, 20, 120, 60, 0, 'red', True)
        sessel=self.modell.Neu(Sessel, 100, 130, 60, 60, 0, 'orange', True)
        sofa=self.modell.Neu(Sofa, 70, 290, 120, 60, 180, 'orange', True)
        ovaler=self.modell.Neu(OvalerTisch, 70, 200, 120, 80, 0, 'brown', True)

### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
